<?php
/**
 * Analyse de la recevalibité d'une liste électorale
 *
 * @plugin Listes électorales (prefix : Listes_electorales)
 * @copyright  2020
 * @author     Vincent CALLIES
 * @licence    GNU/GPL
 * @package SPIP\List_elec\Recevabilites
 */
 
if (!defined("_ECRIRE_INC_VERSION")) return;

/**
 * Fonction d'analyse de la recevabilité d'une liste électorale
 *
 * Cette fonction est appelée par le pipeline affiche_milieu lors de la vue de list_elec
 *
 * Variables significatives utilisées dans la fonction :
 * $id_list_scrutin : identifiant unique du scrutin
 * $id_list_elec : identifiant unique de la liste
 * $script : script de recevabilité de la liste 
 * $nb_total : nombre total des membres de la liste
 * $nb_femmes : nombre de femmes sur la liste
 * $pc_femmes : pourcentage d'femmes sur la liste
 * $nb_hommes : nombre d'hommes sur la liste
 * $pc_hommes : pourcentage d'hommes sur la liste
 *
 * @param  string    $fichier
 *  nécessaire. nom du fichier Yaml, sans l'extension,
 *  contenant les caractéristiques de l'analyse à faire.
 * @return string $sortie
 *  retourne un chaine vide ou formatée avec les informations de recevabilité.
**/
 
function recevabilites_analyse_de_recevabilite_dist($fichier=null,$id_list_elec=null,$id_list_scrutin=null){
	
	if (!$fichier or !$id_list_scrutin or !$id_list_elec) {
		return;
	}

	# Acquisition des membres de la liste triés par groupe puis par position
	if ($liens = sql_allfetsel('*', 'spip_list_membres', array(
			'id_list_elec=' . intval($id_list_elec),
			'statut NOT IN("refuse","poubelle")'
		),'','groupe, position')){ 

		if ($liens) {

			# Aquisition du script en Yaml @link:https://contrib.spip.net/Le-plugin-YAML-v2

			$fichier = find_in_path("recevabilites/${fichier}.yaml");
			include_spip('inc/yaml');
			$script = yaml_decode_file($fichier);
			
			# Rédaction du tableau d’analyse de recevabilité
			
			$tab = '<table class="spip liste" id="recevabilite">';

			# la tête du tableau (thead)

			$nb_total = count($liens);
			$tab .= '<thead><tr><th scope="col">';
			$tab .= 'Élément de recevabilité';
			$tab .= '</th><th scope="col">';
			$tab .= 'Constat';
			$tab .= '</th><th class="picto" scope="col">';
			$tab .= 'Recevabilité';
			$tab .= '</th></tr></thead>';
			
			# le pied du tableau (tfoot)

			$tab .= '<tfoot><tr>';
			$title = '';
			if (isset($script['listes']['global']['minimum']['explication'])) {
				$title .= _T(reforme_idiome($script['listes']['global']['minimum']['explication']));
			}
			if (isset($script['listes']['global']['imperatif']['explication'])) {
				$title .= _T(reforme_idiome($script['listes']['global']['imperatif']['explication']));
			}
			if ($title) {
				$tab .= '<th class="reponse_formulaire explication decale" title="' . $title . '">';
			} else {
				$tab .= '<th>';
			}
			$tab .= 'Total des membres de la liste';
			$tab .= '</th>';
			$tab .= '<th>';
			$nb_total = count($liens);
			$tab .= $nb_total;
			$tab .= '</th>';
			$title = ''; $class = 'success'; $donnee = $nb_total;
			// a-t-on un nombre global à atteindre, impératif ou minimum ?
			if (isset($script['listes']['global']['imperatif']['nb'])) {
				if ($script['listes']['global']['imperatif']['nb'] == $nb_total) {
					$title .= _T(reforme_idiome($script['listes']['global']['imperatif']['success']), array('nb'=> $nb_total));
					$class = 'success';
					$donnee = $nb_total;
				} else {
					$title .= _T(reforme_idiome($script['listes']['global']['imperatif']['error']), array('nb'=> $script['listes']['global']['imperatif']['nb']));
					$class = 'error';
					$donnee = ($nb_total - $script['listes']['global']['imperatif']['nb']);
				}
			} elseif (isset($script['listes']['global']['minimum']['nb'])) {
				foreach ($script['listes']['global'] as $cle => $valeurs) {
					if ($nb_total >= $script['listes']['global'][$cle]['nb']) {
						if ($nb_total == $script['listes']['global'][$cle]['nb'] or (($nb_total > $script['listes']['global'][$cle]['nb']) and ($cle != 'maximum'))) {
							$title .=  _T(reforme_idiome($script['listes']['global'][$cle]['success']), array('nb'=> $script['listes']['global'][$cle]['nb'])) . "\n";
						} elseif ($cle == 'maximum') {
							$title .=  _T(reforme_idiome($script['listes']['global'][$cle]['error']), array('nb'=> $script['listes']['global'][$cle]['nb'])) . "\n";
							$class = 'error';
						}
					} elseif (in_array($cle, array('minimum','maximum'))) {
						if ($cle == 'minimum') {
							$title .= _T(reforme_idiome($script['listes']['global'][$cle]['error']), array('nb'=> $script['listes']['global'][$cle]['nb']));
							$class = 'error';
						} else {
							$title .=  _T(reforme_idiome($script['listes']['global'][$cle]['reste']), array('nb'=> ($script['listes']['global'][$cle]['nb'] - $nb_total))) . "\n";
						}
					}
				}
			}
			# Spéficitées à prendre en compte globalement
			if (isset($script['listes']['specific'])) {
				if (isset($script['listes']['specific']['pair']['type']) and  $script['listes']['specific']['pair']['type'] == '2n') {
					if ($nb_total/2 != floor($nb_total/2)) {
						$title .=  _T(reforme_idiome($script['listes']['specific']['pair']['error']));
						$class = 'error';
					} else {
						$title .=  _T(reforme_idiome($script['listes']['specific']['pair']['success']));
					}
				}
			}
			$tab .= '<th class="reponse_formulaire ' . $class . ' center" title="' . $title . '">' . $donnee . '</th>';
			$tab .= '</tr></tfoot>';

			# le corps du tableau (tbody)

			$tab .= '<tbody>';
			
			# indication de la parité, si la parité à atteindre est définie par un pourcentage
			
			if ($script['listes']['parite']['type']=='def'){
				$nb_femmes = array_count_values(array_column($liens, 'genre'))['femme'];
				$pc_femmes = round($nb_femmes / ($nb_total / 100), 2);
				$nb_hommes = array_count_values(array_column($liens, 'genre'))['homme'];
				$pc_hommes = round($nb_hommes / ($nb_total/100), 2);
				// répartition paritaire minimale au regard des effectifs
				$pc_femmes_effectif = $script['listes']['parite']['pc']['femmes'];
				$pc_hommes_effectif = $script['listes']['parite']['pc']['hommes'];
				$nb_membres_minimum = $script['listes']['global']['minimum']['nb'];
				$nb_femmes_minimum = $nb_membres_minimum / 100 * $pc_femmes_effectif;
				$nb_hommes_minimum = $nb_membres_minimum / 100 * $pc_hommes_effectif;
				// repartition paritaire (recevable) au regard du nombre de candidats dans la liste
				$nb_femmes_recevables = $nb_total / 100 * $pc_femmes_effectif;
				$nb_hommes_recevables = $nb_total / 100 * $pc_hommes_effectif;
				// répartition paritaire maximale au regard de la liste maximale
				$nb_membres_maximum = $script['listes']['global']['maximum']['nb'];
				$nb_femmes_maximum = $nb_membres_maximum / 100 * $pc_femmes_effectif;
				$nb_hommes_maximum = $nb_membres_maximum / 100 * $pc_hommes_effectif;
				
				// divergeance entre la liste et la répartition exigée
				
				$tab .= '<tr>';
				if (isset($script['listes']['parite']['defaut']['explication'])){
					$tab .= '<td class="reponse_formulaire explication decale" title="' . _T(reforme_idiome($script['listes']['parite']['defaut']['explication'])) . '">';
				} else {
					$tab .= '<td>';
				}
				$tab .= 'Femme';
				$tab .= '</td><td>';
				$tab .= $nb_femmes;
				$tab .= '</td>';
				$title = '';
				if (($nb_femmes >= floor($nb_femmes_minimum)) and ($nb_femmes <= ceil($nb_femmes_maximum))) {
					# on a ce qu'il faut pour déposer une liste (en ce qui concerne les femmes)
					$title .= _T(reforme_idiome($script['listes']['parite']['femmes']['success']), array(
							'min' => floor($nb_femmes_minimum),
							'max' => ceil($nb_femmes_maximum)
						)
					);
					# faut-il revoir ce nombre au regard de celui des hommes ?
					if ((($nb_femmes == floor($nb_femmes_recevables)) and ($nb_hommes == ceil($nb_hommes_recevables))) or (($nb_femmes == ceil($nb_femmes_recevables)) and ($nb_hommes == floor($nb_hommes_recevables)))){
						if ($nb_femmes == $nb_femmes_recevables) {
							$title .= "\n" . _T(reforme_idiome($script['listes']['parite']['entier']['success']), array(
									'nb_f' => $nb_femmes_recevables,
									'nb_h' => $nb_hommes_recevables
								)
							);
						} elseif ($nb_femmes == ceil($nb_femmes_recevables)) {
							$title .= "\n" . _T(reforme_idiome($script['listes']['parite']['entier_superieur']['success']), array(
									'nb_f' => $nb_femmes_recevables . ' &uarr; ' . ceil($nb_femmes_recevables),
									'nb_h' => $nb_hommes_recevables . ' &darr; ' . floor($nb_hommes_recevables)
								)
							);
						} elseif ($nb_femmes == floor($nb_femmes_recevables)) {
							$title .= "\n" . _T(reforme_idiome($script['listes']['parite']['entier_inferieur']['success']), array(
									'nb_f' => $nb_femmes_recevables . ' &darr; ' . floor($nb_femmes_recevables),
									'nb_h' => $nb_hommes_recevables . ' &uarr; ' . ceil($nb_hommes_recevables)
								)
							);
						}
						$tab .= '<td class="reponse_formulaire success valign center" title="' . $title . '"  rowspan="2">' . $nb_femmes . ' &#x2640; / &#x2642; ' . $nb_hommes . '</td></tr>'; 
					} else {
						# On pourrait déposer, mais...
					if ($nb_femmes_recevables == floor($nb_femmes_recevables)) {
							$title .= "\n" . _T(reforme_idiome($script['listes']['parite']['entier']['error']), array(
									'total' => $nb_total,
									'nb_f' => $nb_femmes_recevables,
									'nb_h' => $nb_hommes_recevables
								)
							);
						} else {
							$title .= "\n" . _T(reforme_idiome($script['listes']['parite']['entier_superieur']['error']), array(
									'total' => $nb_total,
									'nb_f' => $nb_femmes_recevables . ' &uarr; ' . ceil($nb_femmes_recevables),
									'nb_h' => $nb_hommes_recevables . ' &darr; ' . floor($nb_hommes_recevables)
								)
							);
							$title .= " " . _T(reforme_idiome($script['listes']['parite']['entier_inferieur']['error']), array(
									'nb_f' => $nb_femmes_recevables . ' &darr; ' . floor($nb_femmes_recevables),
									'nb_h' => $nb_hommes_recevables . ' &uarr; ' . ceil($nb_hommes_recevables)
								)
							);
						}
						# le logiciel propose de... (à faire)
						$tab .= '<td class="reponse_formulaire error valign center" title="' . $title . '"  rowspan="2">' . $nb_femmes . ' &#x2640; / &#x2642; ' . $nb_hommes . '</td></tr>'; 
					}
				} else {
					# on ne remplit pas le critère de recevabilité
					$tab .= '<td class="reponse_formulaire error center" title="' . _T(reforme_idiome($script['listes']['parite']['femmes']['error']), array(
							'min' => floor($nb_femmes_minimum),
							'max' => ceil($nb_femmes_maximum)
						)
					) . '">';
					if ($nb_femmes < floor($nb_femmes_minimum)) {
						$tab .= ($nb_femmes - floor($nb_femmes_minimum)) . '</td></tr>';
					}
					if ($nb_femmes > ceil($nb_femmes_maximum)) {
						$tab .= '+' . (ceil($nb_femmes_maximum) - $nb_femmes) . '</td></tr>';
					}
				}
				
				$tab .= '<tr><td>';
				$tab .= 'Homme';
				$tab .= '</td><td>';
				$tab .= $nb_hommes;
				$tab .= '</td></tr>';
			}

			$groupes_en_bdd = array_unique(array_column($liens, 'groupe')); // groupe existant
			$groupes = array();
			foreach ($script['groupes'] as $cle => $valeur) {
				$groupes[] = $cle;
			}
			$nb_groupes = count($groupes_en_bdd);

			# indications sur le nombre de groupes s'il s'agit d'une condition de recevabilité

			if (isset($script['listes']['sections']['imperatif']) and isset($script['listes']['sections']['imperatif']['nb']) and $script['listes']['sections']['imperatif']['nb']>0) {
				$tab .= '<tr>';
				if (isset($script['listes']['sections']['imperatif']) and isset($script['listes']['sections']['imperatif']['explication'])) {
					$tab .= '<td class="reponse_formulaire explication decale" title="' . _T(reforme_idiome($script['listes']['sections']['imperatif']['explication'])) . '">';
				} else {
					$tab .= '<td>';
				}
				$tab .= 'Nbre de groupes';
				$tab .= '</td><td>';
				$tab .= $nb_groupes;
				$tab .= '</td>';
				$nb_groupes_declares = count($groupes);
				if ($script['listes']['sections']['imperatif']['nb'] == $nb_groupes) {
					$tab .= '<td class="reponse_formulaire success center" title="' . _T(reforme_idiome($script['listes']['sections']['imperatif']['success']), array('nb'=> $nb_groupes)) . '">' . $nb_groupes . '</td></tr>';
				} else {
					$tab .= '<td class="error center" title="' . _T(reforme_idiome($script['listes']['sections']['imperatif']['error'])) . '">' . ($nb_groupes - $nb_groupes_declares) . '</td></tr>';
				}
			}

			# indications par groupes

			foreach ($groupes as $cle => $valeur) {
				$tab .= '<tr><td>';
				$tab .= 'Groupe '.$valeur;
				$tab .= '</td>';
				$nbg = $nbgf = $nbgh = $identifiant = 0;$precedent = $alterne = '';
				foreach ($liens as $l => $valeurs) {
					if ($valeurs['groupe'] == $valeur) {
						$nbg++;
						if ($valeurs['genre'] == 'femme') {
							$nbgf++;
							if ($precedent == 'femme') {
								$alterne = 'non'; $erreur_alternance = $identifiant;
							}
							$precedent = 'femme'; $identifiant = $valeurs['id_list_membre'];
						} elseif ($valeurs['genre'] == 'homme') {
							$nbgh++;
							if ($precedent == 'homme') {
								$alterne = 'non'; $erreur_alternance = $identifiant;
							}
							$precedent = 'homme'; $identifiant = $valeurs['id_list_membre'];
						}
					}
				}
				$tab .= '<td>';
				$tab .= $nbg;
				$tab .= '</tb>';
				# nombre obligatoire par groupes
				// un nombre impératif

				if ($nb_imperatif = $script['groupes'][$valeur]['imperatif']['nb']){
					if ($nb_imperatif == $nbg){
						$idiome_precis = $script['groupes'][$valeur]['imperatif']['success'];
						$idiome_defaut = $script['listes']['sections']['imperatif']['defaut']['success'];
						$class = 'success';
						$nombre = $nbg;
						$idiome = $idiome_precis ? $idiome_precis : $idiome_defaut;
						if ($idiome) {
							$inclure = _T(reforme_idiome($idiome), array('nb'=> $nbg));
						}
					} else {
						$idiome_precis = $script['groupes'][$valeur]['imperatif']['error'];
						$idiome_defaut = $script['listes']['sections']['imperatif']['defaut']['error'];
						$class = 'error';
						$nombre = ($nbg - $script['groupes'][$valeur]['imperatif']['nb']);
						if ($nombre > 1) {
							$nombre = '+ ' . $nombre;
						}
						$idiome = $idiome_precis ? $idiome_precis : $idiome_defaut;
						if ($idiome) {
							$inclure = _T(reforme_idiome($idiome), array('nb'=> $nb_imperatif));
						}
					}
					if (!$idiome) {  // on a pas l'idiome pour expliquer et on le dit
						spip_log(_T('list_elec:log_err_absence_idiome', array(
								'fichier' => $fichier,
								'chemin' => 'listes/sections/imperatif/defaut/' . $class,
								'liste' => $id_list_elec,
								'scrutin' => $id_list_scrutin,
							)
						),
							'list_elec.' . _LOG_INFO_IMPORTANTE
						);
						$inclure = _T('list_elec:title_err_absence_idiome');
					}
					$tab .=  '<td class="reponse_formulaire ' . $class . ' center" title="' . $inclure . '">' . $nombre . '</td></tr>';
				} elseif (isset($script['groupes'][$valeur]['si'])) {
				// un nombre impératif... si un contexte particulier est rencontré
					if (isset($script['groupes'][$valeur]['si']['global'])) {
						if ($script['groupes'][$valeur]['si']['global'] == $nb_total) {
							if (isset($script['groupes'][$valeur]['si']['alors'])) {
								if ($script['groupes'][$valeur]['si']['alors'] == 'pair') {
									if (($nbg/2) == ceil($nbg/2)){
										$tab .= '<td class="reponse_formulaire success center" title="' . _T(reforme_idiome($script['groupes'][$valeur]['success'])) . '">' . $nbg . '</td></tr>';
									} else {
										$tab .= '<td class="reponse_formulaire error center" title="' . _T(reforme_idiome($script['groupes'][$valeur]['error'])) . '">' . $nbg . '</td></tr>';
									}
								}
							}
						} else {
							$tab .= '<td class="reponse_formulaire success center" title="' . _T(reforme_idiome($script['groupes'][$valeur]['inapplicable'])) . '">' . $nbg . '</td></tr>';
						}
					}
				} else {
					$tab .= '<td></td></tr>';
				}

				# si alternance ou appréciation globale, pas besoin de calculer par groupe

				if ((!$script['groupes'][$valeur]['alterne']['mode'] == 'groupe' and !$script['listes']['parite']['defaut']['mode']=='groupe')
					and 
					!$script['listes']['parite']['defaut']['mode'] == 'global') {
					$tab .= '<tr><td>';
					$tab .= 'Groupe '.$valeur.' Femmes';
					$tab .= '</td><td>';
					$tab .= $nbgf;
					$tab .= '</td></tr>';
	
					$tab .= '<tr><td>';
					$tab .= 'Groupe '.$valeur.' Hommes';
					$tab .= '</td><td>';
					$tab .= $nbgh;
					$tab .= '</td></tr>';
				}

				# calcul de la parité par alternance par groupe

				if (isset($script['groupes'][$valeur]['alterne']) or (isset($script['listes']['parite']['type']) and $script['listes']['parite']['type'] == 'alterne')) {
					$tab .= '<tr>';
					if ($script['groupes'][$valeur]['alterne']['explication']){
						$tab .= '<td class="reponse_formulaire explication decale" title="' . _T(reforme_idiome($script['groupes'][$valeur]['alterne']['explication'])) . '">';
					} elseif ($script['listes']['parite']['defaut']['explication']) {
						$tab .= '<td class="reponse_formulaire explication decale" title="' . _T(reforme_idiome($script['listes']['parite']['defaut']['explication'])) . '">';
					} else {
						$tab .= '<td>';
					}
					if ($script['groupes'][$valeur]['alterne']['titre']){
						$tab .= _T(reforme_idiome($script['groupes'][$valeur]['alterne']['titre']), array('groupe' => $valeur));
					} elseif ($script['listes']['parite']['defaut']['titre']){
						$tab .= _T(reforme_idiome($script['listes']['parite']['defaut']['titre']), array('groupe' => $valeur));
					} else {
						$tab .= $valeur;
					}
					$tab .= '</td><td>';
					$alterne = $alterne ? 'non' : 'oui';
					if ($nbg > 1){
						$tab .= $alterne;
					}
					$tab .= '</tb>';
					if ($script['groupes'][$valeur]['alterne']['mode'] == 'groupe' or $script['listes']['parite']['defaut']['mode'] == 'groupe'){
						// il est demandé une alternance dans le groupe
						if ($nbg > 1){
							if ($alterne == 'oui') {
								$idiome_precis = $script['groupes'][$valeur]['alterne']['success'];
								$idiome_defaut = $script['listes']['parite']['defaut']['success'];
								$class = 'success';
								$idiome = $idiome_precis ? $idiome_precis : $idiome_defaut;
								if ($idiome) {
									$inclure = _T(reforme_idiome($idiome));
								}
							} else {
								$idiome_precis = $script['groupes'][$valeur]['alterne']['error'];
								$idiome_defaut = $script['listes']['parite']['defaut']['error'];
								$class = 'error';
								$idiome = $idiome_precis ? $idiome_precis : $idiome_defaut;
								if ($idiome) {
									$inclure = _T(reforme_idiome($idiome), array('nb' => $erreur_alternance));
								}
							}
							if (!$idiome) {  // on a pas l'idiome pour expliquer et on le dit
								spip_log(_T('list_elec:log_err_absence_idiome', array(
										'fichier' => $fichier,
										'chemin' => 'listes/parite/defaut/' . $class,
										'liste' => $id_list_elec,
										'scrutin' => $id_list_scrutin,
									)
								),
									'list_elec.' . _LOG_INFO_IMPORTANTE
								);
								$inclure = _T('list_elec:title_err_absence_idiome');
							}
							$tab .= '<td class="reponse_formulaire ' . $class . ' center" title="' . $inclure . '">&#x2640; &#x2642;</td></tr>';
						}
					}
				}

			}
			$tab .= '</tbody></table>';
		}
	}

	$texte = $tab . '<br class="nettoyeur">';

	return $texte;
}

